# Steam Pipe Dreams
# Copyright (C) 2006 Andrew Wilkinson <ajw140@york.ac.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import pygame

import main
from pieces import Piece, SmokeController
from gfx import loadImage

class Board:
    def __init__(self):
        self.boardimg = loadImage("board.png")
        for x in range(18):
            pygame.draw.line(self.boardimg, (0, 0, 0), (x*32, 0), (x*32, 17*32), 1)
        for y in range(18):
            pygame.draw.line(self.boardimg, (0, 0, 0), (0, y*32), (17*32, y*32), 1)
        self.rect = pygame.Rect(228, 28, 17*32, 17*32)

        self.smoke = SmokeController()

        self._board = [[None for _ in range(17)] for _ in range(17)]

    def setPiece(self, x, y, piece):
        assert isinstance(x, int) and isinstance(y, int) and x >= 0 and x < 17 and y >= 0 and y < 17 and isinstance(piece, Piece)
        if self._board[y][x] and not self._board[y][x].pipe:
            return
        self._board[y][x] = piece
        piece.rect.topleft = (x*32 + 228, y*32 + 28)

        if y > 0 and self._board[y-1][x] is not None:
            piece.up = self._board[y-1][x]
            self._board[y-1][x].down = piece
        if y < 16 and self._board[y+1][x] is not None:
            piece.down = self._board[y+1][x]
            self._board[y+1][x].up = piece
        if x > 0 and self._board[y][x-1] is not None:
            piece.left = self._board[y][x-1]
            self._board[y][x-1].right = piece
        if x < 16 and self._board[y][x+1] is not None:
            piece.right = self._board[y][x+1]
            self._board[y][x+1].left = piece
        self.updateGuages()

    def deletePiece(self, x, y):
        assert isinstance(x, int) and isinstance(y, int) and x >= 0 and x < 17 and y >= 0 and y < 17
        if self._board[y][x] and not self._board[y][x].pipe:
            return
        self._board[y][x] = None

        if y > 0 and self._board[y-1][x] is not None:
            self._board[y-1][x].down = None
        if y < 16 and self._board[y+1][x] is not None:
            self._board[y+1][x].up = None
        if x > 0 and self._board[y][x-1] is not None:
            self._board[y][x-1].right = None
        if x < 16 and self._board[y][x+1] is not None:
            self._board[y][x+1].left = None
        self.updateGuages()

    def update(self, time):
        for row in self._board:
            for col in row:
                if col is not None:
                    col.update(time)
        self.smoke.update(time)

    def render(self, screen):
        screen.blit(self.boardimg, self.rect)
        for row in self._board:
            for col in row:
                if col is not None:
                    col.render(screen)
        self.smoke.render(screen)

    def click(self, pos, button):
        # it's not on the board so ignore it!
        if not self.rect.collidepoint(pos):
            return False

        if main.gm.cursor.object and button == 1:
            x, y = self.calcSquare(pos)
            if self._board[y][x] is not None:
                if not self._board[y][x].pipe:
                    return True
            main.gm.score -= main.gm.cursor.object.points
            self.setPiece(x, y, main.gm.cursor.object.__class__())
        elif main.gm.cursor.object and button == 3:
                obj = main.gm.cursor.drop()
                obj = obj.rotate()
                main.gm.cursor.pickUp(obj)
        elif main.gm.cursor.type == main.gm.cursor.BOMB:
            x, y = self.calcSquare(pos)
            if self._board[y][x] is not None:
                self.deletePiece(x, y)
        elif main.gm.cursor.type == main.gm.cursor.SPANNER and button == 1:
            x, y = self.calcSquare(pos)
            if self._board[y][x] is not None:
                self._board[y][x].adjustUp()
        elif main.gm.cursor.type == main.gm.cursor.SPANNER and button == 3:
            x, y = self.calcSquare(pos)
            if self._board[y][x] is not None:
                self._board[y][x].adjustDown()
        return True

    def calcSquare(self, pos):
        x, y = pos
        x -= 228
        y -= 28
        return (x/32, y/32)

    def updateGuages(self):
        for x in range(17):
            for y in range(17):
                if self._board[y][x] is None:
                    continue
                self._board[y][x].graph = False
        for x in range(17):
            for y in range(17):
                if self._board[y][x] is None or self._board[y][x].no_graph:
                    continue
                if self._board[y][x].require_graph:
                    self._board[y][x].graph = True
                    continue
                if x > 0 and self._board[y][x-1] is not None and (self._board[y][x-1].require_graph or self._board[y][x-1].graph):
                    continue
                if x < 16 and self._board[y][x+1] is not None and (self._board[y][x+1].require_graph or self._board[y][x+1].graph):
                    continue
                if y > 0 and self._board[y-1][x] is not None and (self._board[y-1][x].require_graph or self._board[y-1][x].graph):
                    continue
                if y < 16 and self._board[y+1][x] is not None and (self._board[y+1][x].require_graph or self._board[y+1][x].graph):
                    continue
                self._board[y][x].graph = True

    def tooltip(self, pos):
        # it's not on the board so ignore it!
        if not self.rect.collidepoint(pos):
            return None

        x, y = self.calcSquare(pos)
        if self._board[y][x] is not None:
            return self._board[y][x].tooltip
